/*****************************************************************
   OtBbNfoCX
 *****************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <windows.h>
#include "gdevlib.h"

#define TEXTHEIGHT 13
#define TEXTWIDTH   7

/* gdev configuration */
static int doublebuf = 0;   /* 1: draw back then copy to front */

static LRESULT CALLBACK window_proc(HWND hWnd, UINT wMsg,
				    WPARAM wParam, LPARAM lParam)
{
  switch (wMsg){
  case WM_PAINT:
    BitBlt(gd->hdc, 0, 0, gd->width, gd->height, gd->hdc_back, 0, 0, SRCCOPY);
    return DefWindowProc(hWnd, wMsg, wParam, lParam);
  case WM_CLOSE:
    return(0) ;
    //    DestroyWindow(hWnd);
    //    PostQuitMessage(0);
    //    exit(0);
  default:
    return DefWindowProc(hWnd, wMsg, wParam, lParam);
  }
}

Gdev *OpenGdev(int width,int height, char *title)
{
  Gdev *gd ;
  int w, h;
  RECT rect;
  WNDCLASS wclass;
  DWORD wstyle = (0)? WS_OVERLAPPED|WS_CAPTION|WS_SYSMENU|WS_MINIMIZEBOX
                      : WS_OVERLAPPEDWINDOW;
  //DWORD wstyle =  WS_OVERLAPPED|WS_CAPTION|WS_SYSMENU|WS_MINIMIZEBOX ;

  gd=malloc(sizeof(Gdev)) ;
  if( gd==NULL ) return(gd) ;
  
  wclass.lpszClassName ="Display";
  wclass.lpszMenuName  = NULL;
  wclass.hInstance     = GetModuleHandle(NULL);
  wclass.lpfnWndProc   = window_proc;
  wclass.hCursor       = LoadCursor(NULL, IDC_ARROW);
  wclass.hIcon         = NULL;
  wclass.hbrBackground = GetStockObject(BLACK_BRUSH);
  wclass.style         = 0;
  wclass.cbClsExtra    = 0;
  wclass.cbWndExtra    = 0;
  
  if (!RegisterClass(&wclass)){
    free(gd) ;
    return(NULL) ;
  }
  
  gd->width = width ;
  gd->height= height;
  SetRect(&rect, 0, 0, gd->width, gd->height);
  AdjustWindowRect(&rect, wstyle, 0);
  w = rect.right  - rect.left;
  h = rect.bottom - rect.top;
  
  gd->win = CreateWindow(wclass.lpszClassName, title, wstyle,
		         CW_USEDEFAULT, CW_USEDEFAULT,
		         w, h, NULL, NULL, wclass.hInstance, NULL);
  ShowWindow(gd->win, SW_SHOWDEFAULT);
  
  /* set as default */
  gd->hdc = GetDC(gd->win);
  
  /* set up background bitmap */
  gd->hdc_back = CreateCompatibleDC(gd->hdc);
  
  gd->bi.bmiHeader.biSize             = sizeof(BITMAPINFOHEADER);
  gd->bi.bmiHeader.biWidth            = 640 ;
  gd->bi.bmiHeader.biHeight           = -512;
  gd->bi.bmiHeader.biPlanes           = 1;
  gd->bi.bmiHeader.biBitCount         = 32;
  gd->bi.bmiHeader.biCompression      = BI_RGB;
  gd->bi.bmiHeader.biSizeImage        = 0;
  gd->bi.bmiHeader.biXPelsPerMeter    = 0;
  gd->bi.bmiHeader.biYPelsPerMeter    = 0;
  gd->bi.bmiHeader.biClrUsed          = 1;
  gd->bi.bmiHeader.biClrImportant     = 0;
  gd->bmp_back = CreateDIBSection( gd->hdc, &gd->bi, 0, &gd->vram, NULL, 0 );
  SelectObject(gd->hdc_back, gd->bmp_back);
  
  /* font */
  gd->hfont = CreateFont(TEXTHEIGHT, TEXTWIDTH, 0, 0, 
			 0, FALSE, FALSE, FALSE, DEFAULT_CHARSET,
			 OUT_DEFAULT_PRECIS,CLIP_TT_ALWAYS,PROOF_QUALITY,
			 FIXED_PITCH|FF_ROMAN, "TERMINAL");
  SelectObject(gd->hdc, gd->hfont);
  SelectObject(gd->hdc_back, gd->hfont);
  
  FlushConsoleInputBuffer(GetStdHandle(STD_INPUT_HANDLE));

  return(gd) ;
}

void FlushBuffer(Gdev *gd)
{
  GdiFlush();
  if (doublebuf)
    BitBlt(gd->hdc, 0, 0, gd->width, gd->height, gd->hdc_back, 0, 0, SRCCOPY);
  else 
    BitBlt(gd->hdc_back, 0, 0, gd->width, gd->height, gd->hdc, 0, 0, SRCCOPY);
}

void CloseGdev(Gdev *gd)
{
  if (gd->win) {
    DeleteDC(gd->hdc);
    DeleteDC(gd->hdc_back);
    DeleteObject(gd->bmp_back);
    if (gd->current_pen) DeleteObject(gd->current_pen);
    if (gd->current_brush) DeleteObject(gd->current_brush);
    UnregisterClass("Display", GetModuleHandle(NULL));
    gd->win = 0;
  }
}

void ClearGdev(Gdev *gd)
{
  HBRUSH new_brush;
  COLORREF bg_color ;
  
  bg_color=RGB(0,0,0) ;
  new_brush = CreateSolidBrush(bg_color);
  SelectObject(gd->hdc, new_brush);
  if (gd->current_brush) DeleteObject(gd->current_brush);
  gd->current_brush = new_brush;
  Rectangle(gd->hdc, -10, -10, gd->width+10, gd->height+10);
}

void DrawPix(Gdev *gd, int x, int y, COLORREF pix)
{
  SetPixel(gd->hdc_back, x, y, pix);
}
